@@ -20,6 +20,7 @@
 #
 # Author(s): Lorenzo Gil Sanchez <lgs@sicem.biz>
 #            Johan Dahlin <jdahlin@async.com.br>
+#            Ronaldo Maia <romaia@async.com.br>
 #
 
 """Data type converters with locale and currency support.
@@ -45,6 +46,7 @@ import time
 
 from kiwi import ValueUnset
 from kiwi.enums import Alignment
+from kiwi.python import enum
 
 try:
     from decimal import Decimal, InvalidOperation
@@ -109,6 +111,14 @@ class ConverterRegistry:
         del self._converters[ctype]
 
     def get_converter(self, converter_type):
+        # This is a hack:
+        # If we're a subclass of enum, create a dynamic subclass on the
+        # fly and register it, it's necessary for enum.from_string to work.
+        if (issubclass(converter_type, enum) and
+            not converter_type in self._converters):
+            self.add(type(enum.__class__.__name__ + 'EnumConverter',
+                       (_EnumConverter,), dict(type=converter_type)))
+
         try:
             return self._converters[converter_type]
         except KeyError:
@@ -117,7 +127,7 @@ class ConverterRegistry:
     def get_converters(self, base_classes=None):
         if base_classes is None:
             return self._converters.values()
-        
+
         converters = []
         if object in base_classes:
             #: Ugly, but cannot remove from tuple!
@@ -397,17 +407,23 @@ DATE_REPLACEMENTS_WIN32 = [
     (re.compile('yy?'), '%y')
     ]
 
+# This "table" contains, associated with each
+# key (ie. strftime "conversion specifications")
+# a tuple, which holds in the first position the
+# mask characters and in the second position
+# a "human-readable" format, used for outputting user
+# messages (see method _BaseDateTimeConverter.convert_format) 
 DATE_MASK_TABLE = {
-    '%m': '00',
-    '%y': '00',
-    '%d': '00',
-    '%Y': '0000',
-    '%H': '00',
-    '%M': '00',
-    '%S': '00',
-    '%T': '00:00:00',
+    '%m': ('00', _('mm')),
+    '%y': ('00', _('yy')),
+    '%d': ('00', _('dd')),
+    '%Y': ('0000', _('yyyy')),
+    '%H': ('00', _('hh')),
+    '%M': ('00', _('mm')),
+    '%S': ('00', _('ss')),
+    '%T': ('00:00:00', _('hh:mm:ss')),
     # FIXME: locale specific
-    '%r': '00:00:00 LL',
+    '%r': ('00:00:00 LL', _('hh:mm:ss LL')),
     }
 
 class _BaseDateTimeConverter(BaseConverter):
@@ -487,7 +503,7 @@ class _BaseDateTimeConverter(BaseConverter):
     def get_mask(self):
         mask = self.get_format()
         for format_char, mask_char in DATE_MASK_TABLE.items():
-            mask = mask.replace(format_char, mask_char)
+            mask = mask.replace(format_char, mask_char[0])
 
         return mask
 
@@ -501,6 +517,13 @@ class _BaseDateTimeConverter(BaseConverter):
 
         return value.strftime(format)
 
+    def _convert_format(self, format):
+        "Convert the format string to a 'human-readable' format"
+        for char in DATE_MASK_TABLE.keys():
+            format = format.replace(char, DATE_MASK_TABLE[char][1])
+
+        return format
+
     def from_string(self, value):
         "Convert a string to a date"
 
@@ -520,7 +543,7 @@ class _BaseDateTimeConverter(BaseConverter):
         except ValueError:
             raise ValidationError(
                 _('This field requires a date of the format "%s" and '
-                  'not "%s"') % (format, value))
+                  'not "%s"') % (self._convert_format(format), value))
 
 class _TimeConverter(_BaseDateTimeConverter):
     type = datetime.time
@@ -575,6 +598,26 @@ class _ObjectConverter(BaseConverter):
     from_string = None
 converter.add(_ObjectConverter)
 
+class _EnumConverter(BaseConverter):
+    type = enum
+    name = _('Enum')
+
+    def as_string(self, value, format=None):
+        if not isinstance(value, self.type):
+            raise ValidationError(
+                "value must be an instance of %s, not %r" % (
+                self.type, value))
+        return value.name
+
+    def from_string(self, value):
+        names = self.type.names
+        if not value in names:
+            raise ValidationError(
+                "Invalid value %s for enum %s" % (value, self.type))
+        return names[value]
+
+converter.add(_EnumConverter)
+
 def lformat(format, value):
     """Like locale.format but with grouping enabled"""
     return locale.format(format, value, 1)
